#include "defs.h"

#include <linux/input.h>
#include <glob.h>
#include <errno.h>
#include <err.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>


static int device = -1;
static int grabbed = 0;

static int read_event(struct input_event *event)
{
	int res;

	res = event->value;

	res = TEMP_FAILURE_RETRY(read(device, event, sizeof(*event)));
	if (res < 0) {
		perror("can not read event device");
		return -errno;
	}

	return 0;
}

static int decode_event(struct input_event *event, struct input_data *in)
{
	int state;

	switch (event->type) {
	case EV_SYN:
		switch (event->code) {
		case SYN_REPORT:
			return 1;
		}
	case EV_KEY:
		state = event->value ? 1 : 0;
		switch (event->code) {
		case BTN_LEFT:
			in->buttons.left = state;
			break;
		case BTN_RIGHT:
			in->buttons.right = state;
			break;
		case BTN_MIDDLE:
			in->buttons.middle = state;
			break;
		case BTN_TOOL_FINGER:
			break;
		case BTN_TOUCH:
			in->touched = state;
			break;
		}
		break;
	case EV_ABS:
		switch (event->code) {
		case ABS_X:
			in->X = event->value;
			break;
		case ABS_Y:
			in->Y = event->value;
			break;
		case ABS_PRESSURE:
			in->P = event->value;
			break;
		}
		break;
	}

	return 0;
}

static int read_data(struct input_data *in)
{
	struct input_event event;
	int res = 0;

	if (device < 0)
		return -ENODEV;

	while (!(res = read_event(&event))) {
		if (decode_event(&event, in))
			break;
	}

	return res;
}

static void cleanup(void)
{
	int res;

	if (device < 0)
		return;

	if (grabbed) {
		res = TEMP_FAILURE_RETRY(ioctl(device, EVIOCGRAB, (int *)0));
		if (res < 0)
			perror("can not release event device");
	}
	res = TEMP_FAILURE_RETRY(close(device));
	if (res < 0)
		perror("can not close event device");
}

#define EVDEV_PATTERN "/dev/input/event*"
#define STICK_NAME "Synaptics USB Styk"

static int is_stick(int dev)
{
	struct input_id id;
	char name[32];
	int res;

	res = TEMP_FAILURE_RETRY(ioctl(dev, EVIOCGID, &id));
	if (res < 0) {
		perror("can not get ID of event device");
		return 0;
	}
	if ( (id.bustype != BUS_USB) ||
		(id.vendor != 0x06cb) )
		return 0;
	if ( !ALLOW_TP &&
		(id.product != 0x0007) &&
		(id.product != 0x0009) )
		return 0;

	res = TEMP_FAILURE_RETRY(ioctl(dev, EVIOCGNAME(31), &name));
	if (res < 0) {
		perror("can not get name of event device");
		return 0;
	}
	if (strcmp(name, STICK_NAME) && !ALLOW_TP)
		return 0;

	res = TEMP_FAILURE_RETRY(ioctl(dev, EVIOCGRAB, (int *)1));
	if (res < 0) {
		perror("can not grab event device");
		return 0;
	}
	grabbed = 1;

	return 1;
}

static int init_device(void)
{
	glob_t file_list;
	int i, res;

	res = glob(EVDEV_PATTERN, GLOB_ERR, NULL, &file_list);
	if (res) {
		perror("can not get list of event devices");
		res = -errno;
		goto exit;
	}

	for (i = 0; i < file_list.gl_pathc; i++) {
		res = TEMP_FAILURE_RETRY(open(file_list.gl_pathv[i], O_RDONLY));
		if (res < 0) {
			warn("can not open %s", file_list.gl_pathv[i]);
			continue;
		}
		if (is_stick(res)) {
			printf("found stick device: %s\n", file_list.gl_pathv[i]);
			device = res;
			res = 0;
			goto exit;
		}
		res = TEMP_FAILURE_RETRY(close(res));
		if (res < 0) {
			warn("can not close %s", file_list.gl_pathv[i]);
			res = -errno;
			goto exit;
		}
	}
	res = -ENODEV;
	fprintf(stderr, "no stick event device found!\n");
exit:
	globfree(&file_list);
	return res;
}

struct input_functions input_evdev = {
	.init = init_device,
	.exit = cleanup,
	.get_data = read_data
};
